﻿using System;
using System.ComponentModel;
using System.Windows.Input;

namespace CommandDemo
{
    public class GreetingsCommand : ICommand
    {
        private Func<bool> canExecuteHandler = () => false;
        private Action<string> executeHandler = obj => {};
        public event EventHandler CanExecuteChanged = (sender, e) =>{};

        public event Func<bool> CanExecuteHandler
        {
            add
            {
                canExecuteHandler += value;
                CanExecuteChanged(this, EventArgs.Empty);
            }
            remove
            {
                canExecuteHandler -= value;
                CanExecuteChanged(this, EventArgs.Empty);
            }
        }

        public event Action<string> ExecuteHandler
        {
            add
            {
                executeHandler += value;
            }
            remove
            {
                executeHandler -= value;
            }
        }

        public void Execute(object parameter)
        {
            Delegate[] delegates = executeHandler.GetInvocationList();
            
            foreach(Action<string> act in delegates)
            {
                act.Invoke(parameter != null ? parameter.ToString() : string.Empty);
            }
        }

        public bool CanExecute(object parameter)
        {
            Delegate[] delegates = canExecuteHandler.GetInvocationList();

            foreach (Func<bool> fun in delegates)
            {
                if (fun.Invoke())
                {
                    return true;
                }
            }

            return false;
        }


        public void RaiseCanExecuteChanged()
        {
            CanExecuteChanged(this, EventArgs.Empty);
        }

    }
}
