﻿using System.ComponentModel;
using System.Windows;

namespace CommandDemo
{
    public class SimplePresenter : DependencyObject
    {
        public static readonly DependencyProperty GreetingsProperty =
            DependencyProperty.Register("Greetings", typeof (GreetingsCommand), typeof (SimplePresenter),
                                        new UIPropertyMetadata(null));

        public SimplePresenter()
        {
            Model = new SimpleModel();
            Model.PropertyChanged += Model_PropertyChanged;

            Greetings = new GreetingsCommand();
        }

        void Model_PropertyChanged(object sender,PropertyChangedEventArgs e)
        {
            if(string.Equals(e.PropertyName,"Greetings"))
            {
                Greetings.RaiseCanExecuteChanged();
            }
            
        }

        public ISimpleModel Model { get; set; }

        public IView View { get; set; }

        public GreetingsCommand Greetings
        {
            get { return (GreetingsCommand) GetValue(GreetingsProperty); }
            set { SetValue(GreetingsProperty, value); }
        }

        private static void SayHello(string hello)
        {
            MessageBox.Show(hello);
        }

        public void AddGreetingsCommands()
        {
            Greetings.CanExecuteHandler += Greetings_CanExecuteHandler;
            Greetings.ExecuteHandler += Greetings_Execute;
        }

        private bool Greetings_CanExecuteHandler()
        {
            return !string.IsNullOrEmpty(Model.Greetings);
        }

        private static void Greetings_Execute(string greetings)
        {
            SayHello(greetings);
        }
    }
}