﻿using System;
using System.Windows;
using System.Windows.Input;

namespace WpfCommandDemo
{
    public class SimplePresenter : DependencyObject
    {
        public static readonly DependencyProperty GreetingsProperty =
            DependencyProperty.Register("Greetings", typeof (RoutedCommand), typeof (SimplePresenter),
                                        new UIPropertyMetadata(null));

        public SimplePresenter()
        {
            Model = new SimpleModel();
            Greetings = new RoutedCommand();
        }

        public ISimpleModel Model { get; set; }

        public IView View { get; set; }

        public RoutedCommand Greetings
        {
            get
            {
                return (RoutedCommand)GetValue(GreetingsProperty);
            }
            set
            {
                SetValue(GreetingsProperty, value);
            }
        }

        private void SayHello(string hello)
        {
            MessageBox.Show(hello);
        }

        public void AddGreetingsCommandsBinding(UIElement element)
        {
            var cb = new CommandBinding(Greetings, Greetings_Execute, Greetings_CanExecuteHandler);
            element.CommandBindings.Add(cb);
        }

        private void Greetings_CanExecuteHandler(Object sender, CanExecuteRoutedEventArgs e)
        {
             e.CanExecute = !string.IsNullOrEmpty(Model.Greetings);
        }

        private void Greetings_Execute(Object sender, ExecutedRoutedEventArgs e)
        {
            SayHello(e.Parameter.ToString());
        }
    }
}